<?php
/**
 * Ce fichier contient la configuration et l'ensemble des fonctions implémentant le service Méteo-Concept (meteoc).
 * Ce service est capable de fournir des données au format JSON ou XML.
 *
 * @package SPIP\RAINETTE\SERVICES\METEOC
 */
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}


// ------------------------------------------------------------------------------------------------
// Les fonctions qui suivent définissent l'API standard du service et sont appelées par la fonction
// unique de chargement des données météorologiques `meteo_charger()`.
// ------------------------------------------------------------------------------------------------

/**
 * Construit l'url de la requête correspondant au lieu, au type de données et à la configuration utilisateur
 * du service (par exemple, le code d'inscription, le format des résultats...).
 *
 * @api
 * @uses lieu_normaliser()
 *
 * @param string $lieu
 *        Lieu pour lequel on acquiert les données météorologiques.
 * @param string $mode
 *        Type de données météorologiques. Les valeurs possibles sont `infos`, `conditions` ou `previsions`.
 * @param int    $periodicite
 *        La périodicité horaire des prévisions :
 *        - `24`, `12`, `6`, `3` ou `1`, pour le mode `previsions`
 *        - `0`, pour les modes `conditions` et `infos`
 * @param array  $configuration
 *        Configuration complète du service, statique et utilisateur.
 *
 * @return string
 *        URL complète de la requête.
 */
function meteoc_service2url($lieu, $mode, $periodicite, $configuration) {

	// On normalise le lieu et on récupère son format.
	// Le service accepte les coordonnées et la code INSEE
	$lieu_normalise = lieu_normaliser($lieu, $format_lieu);
	if ($format_lieu == 'latitude_longitude') {
		$query = "latlng=${lieu_normalise}";
	} else {
		// Format INSEE
		$query = "insee=${lieu_normalise}";
	}

	// On construit le mode
	if ($mode === 'infos') {
		$demande = 'location/city';
		// On demande de considérer les pays étrangers si on identifie la ville par ses coordonnées
		if ($format_lieu == 'latitude_longitude') {
			$query .= '&world=true';
		}
	} elseif ($mode === 'conditions') {
		$demande = 'observations/around';
	} else {
		$demande = 'forecast/daily';
	}

	return
		$configuration['endpoint_requete']
		. $demande
		. '?token=' . $configuration['inscription']
		. '&' . $query;
}


/**
 * @param array $erreur
 *
 * @return bool
 */
function meteoc_erreur_verifier($erreur) {

	// Initialisation
	$est_erreur = false;

	// Une erreur est uniquement décrite par un message.
	if (!empty($erreur['message'])) {
		$est_erreur = true;
	}

	return $est_erreur;
}


/**
 * Complète par des données spécifiques au service le tableau des informations issu
 * uniquement de la lecture du flux.
 *
 * @api
 *
 * @param array $tableau
 *        Tableau standardisé des conditions contenant uniquement les données fournies sans traitement
 *        par le service.
 * @param array $configuration
 *        Configuration complète du service, statique et utilisateur.
 *
 * @return array
 *        Tableau standardisé des conditions météorologiques complété par les données spécifiques
 *        du service.
 */
function meteoc_complement2infos($tableau, $configuration) {

	if ($tableau) {
		if (empty($tableau['pays'])) {
			// France : le pays n'est pas précisé, on le rajoute
			$tableau['pays'] = 'France';
			$tableau['pays_iso2'] = 'FR';
		} else {
			// Pays limitrophes : Belgique, Luxembourg et Andorre. On rajoute l'ISO alpha-2
			$pays = strtolower($tableau['pays']);
			switch($pays) {
				case 'belgique':
					$tableau['pays_iso2'] = 'BE';
					break;
				case 'luxembourg':
					$tableau['pays_iso2'] = 'LU';
					break;
				case 'andorre':
					$tableau['pays_iso2'] = 'AD';
					break;
			}
		}
	}

	return $tableau;
}


/**
 * Complète par des données spécifiques au service le tableau des conditions issu
 * uniquement de la lecture du flux.
 *
 * @api
 *
 * @param array $tableau
 *        Tableau standardisé des conditions contenant uniquement les données fournies sans traitement
 *        par le service.
 * @param array $configuration
 *        Configuration complète du service, statique et utilisateur.
 *
 * @return array
 *        Tableau standardisé des conditions météorologiques complété par les données spécifiques
 *        du service.
 */
function meteoc_complement2conditions($tableau, $configuration) {

	if ($tableau) {
		include_spip('inc/rainette_convertir');
		// Calcul de la direction du vent (16 points), celle-ci n'étant pas fournie nativement par owm
		$tableau['direction_vent'] = angle2direction($tableau['angle_vent']);

		// Calcul de la température ressentie
		$tableau['temperature_ressentie'] = temperature2ressenti(
			$tableau['temperature_reelle'],
			$tableau['vitesse_vent']
		);

		// Conversion en système impérial
		// --> Etant donné que le service ne concerne que les pays France, Luxembourg, Belgique et Andorre, on
		//     ne propose pas la conversion en système impérial.

		// Compléter le tableau standard avec les états météorologiques calculés
		etat2resume_meteoc($tableau, $configuration);
	}

	return $tableau;
}


/**
 * Complète par des données spécifiques au service le tableau des prévisions issu
 * uniquement de la lecture du flux.
 *
 * @api
 *
 * @param array $tableau
 *        Tableau standardisé des conditions contenant uniquement les données fournies sans traitement
 *        par le service.
 * @param array $configuration
 *        Configuration complète du service, statique et utilisateur.
 * @param int   $index_periode
 *        Index où trouver et ranger les données. Cet index n'est pas utilisé pour les conditions
 *
 * @return array
 *        Tableau standardisé des conditions météorologiques complété par les données spécifiques
 *        du service.
 */
function meteoc_complement2previsions($tableau, $configuration, $index_periode) {

	if (($tableau) and ($index_periode > -1)) {
		include_spip('inc/rainette_convertir');
		// Calcul de la direction du vent (16 points), celle-ci n'étant pas fournie nativement par owm
		$tableau['direction_vent'] = angle2direction($tableau['angle_vent']);

		// Conversion en système impérial
		// --> Etant donné que le service ne concerne que les pays France, Luxembourg, Belgique et Andorre, on
		//     ne propose pas la conversion en système impérial.

		// Compléter le tableau standard avec les états météorologiques calculés
		etat2resume_meteoc($tableau, $configuration);
	}

	return $tableau;
}


/**
 * ---------------------------------------------------------------------------------------------
 * Les fonctions qui suivent sont des utilitaires uniquement appelées par les fonctions de l'API
 * ---------------------------------------------------------------------------------------------
 */

/**
 * Calcule les états en fonction des états météorologiques natifs fournis par le service.
 *
 * @internal
 *
 * @param array $tableau
 *        Tableau standardisé des conditions contenant uniquement les données fournies sans traitement
 *        par le service. Le tableau est mis à jour et renvoyé à l'appelant.
 * @param array $configuration
 *        Configuration complète du service, statique et utilisateur.
 *
 * @return void
 */
function etat2resume_meteoc(&$tableau, $configuration) {

	if ($tableau['code_meteo']) {
		// Determination de l'indicateur jour/nuit qui permet de choisir le bon icone
		// Pour ce service aucun indicateur n'est disponible, on positionne la période à 0 tout le temps
		$tableau['periode'] = 0;

		// Détermination du résumé à afficher:
		// - Meteo-Concept ne fourni pas de résumé dans l'API, seul le code est fourni. Rainette construit donc
		//   un fichier de langue spécifique initialisé en français par le tableau fourni par le service.
		$tableau['resume'] = _T('meteoc:meteo_' . $tableau['code_meteo']);

		// Determination de l'icone qui sera affiché.
		// -- on stocke le code afin de le fournir en alt dans la balise img
		$tableau['icone'] = array();
		$tableau['icone']['code'] = $tableau['code_meteo'];
		// -- on calcule le chemin complet de l'icone.
		if ($configuration['condition'] == $configuration['alias']) {
			// On affiche l'icône natif fourni par le service et désigné par son url
			// en faisant une copie locale dans IMG/.
			// --> pas disponible actuellement pour Méteo-Concept
		} else {
			include_spip('inc/rainette_normaliser');
			if ($configuration['condition'] == "{$configuration['alias']}_local") {
				// On affiche un icône d'un thème local compatible avec Méteo-Concept.
				// --> pas disponible actuellement pour Méteo-Concept
			} else {
				// On affiche l'icône correspondant au code météo transcodé dans le système weather.com.
				$chemin = icone_weather_normaliser(
					$tableau['code_meteo'],
					$configuration['theme_weather'],
					$configuration['transcodage_weather'],
					$tableau['periode']
				);
			}
			include_spip('inc/utils');
			$tableau['icone']['source'] = find_in_path($chemin);
		}
	}
}
